package de.intevation.flys.artifacts.states;

import java.io.File;

import org.w3c.dom.Element;

import org.apache.log4j.Logger;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.artifacts.common.utils.XMLUtils.ElementCreator;

import de.intevation.flys.model.DGM;
import de.intevation.flys.model.River;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.utils.FLYSUtils;


/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class DGMSelect extends DefaultState {

    private static final Logger logger = Logger.getLogger(DGMSelect.class);

    public static final String ERR_EMPTY         = "error_no_dgm_selected";
    public static final String ERR_INVALID_DGM   = "error_invalid_dgm_selected";
    public static final String ERR_BAD_DGM_RANGE = "error_bad_dgm_range";
    public static final String ERR_BAD_DGM_RIVER = "error_bad_dgm_river";


    @Override
    protected String getUIProvider() {
        return "dgm_datacage_panel";
    }


    @Override
    protected Element createStaticData(
        FLYSArtifact   flys,
        ElementCreator creator,
        CallContext    cc,
        String         name,
        String         value,
        String         type
    ) {
        Element dataElement = creator.create("data");
        creator.addAttr(dataElement, "name", name, true);
        creator.addAttr(dataElement, "type", type, true);

        Element itemElement = creator.create("item");
        creator.addAttr(itemElement, "value", value, true);

        creator.addAttr(itemElement, "label", getLabel(cc, value), true);
        dataElement.appendChild(itemElement);

        return dataElement;
    }


    public static String getLabel(CallContext cc, String value) {
        logger.debug("Create label for value: " + value);

        try {
            DGM dgm = DGM.getDGM(Integer.parseInt(value));

            File file = new File(dgm.getPath());
            return file.getName();
        }
        catch (NumberFormatException nfe) {
            logger.warn("Cannot parse int value: '" + value + "'");
        }

        return "";
    }


    @Override
    public boolean validate(Artifact artifact)
    throws IllegalArgumentException
    {
        FLYSArtifact flys = (FLYSArtifact) artifact;

        DGM dgm = getDGM(flys);

        if (dgm == null) {
            throw new IllegalArgumentException(ERR_INVALID_DGM);
        }

        double l = dgm.getLower().doubleValue();
        double u = dgm.getUpper().doubleValue();

        double[] range = FLYSUtils.getKmFromTo(flys);

        if (range[0] < l || range[0] > u || range[1] < l || range[1] > u) {
            throw new IllegalArgumentException(ERR_BAD_DGM_RANGE);
        }

        River selectedRiver = FLYSUtils.getRiver(flys);
        River dgmRiver      = dgm.getRiver();

        if (selectedRiver != dgmRiver) {
            throw new IllegalArgumentException(ERR_BAD_DGM_RIVER);
        }

        return true;
    }


    /**
     * Returns the DGM specified in the parameters of <i>flys</i>.
     *
     * @param flys The FLYSArtifact that knows the ID of a DGM.
     *
     * @throws IllegalArgumentException If the FLYSArtifact doesn't know the ID
     * of a DGM.
     *
     * @return the DGM specified by FLYSArtifact's parameters.
     */
    public static DGM getDGM(FLYSArtifact flys)
    throws IllegalArgumentException
    {
        try {
            Integer dgmId = flys.getDataAsInteger("dgm");
            if (dgmId == null) {
                throw new IllegalArgumentException(ERR_EMPTY);
            }

            logger.debug("Found selected dgm: '" + dgmId + "'");

            return DGM.getDGM(dgmId);
        }
        catch (NumberFormatException nfe) {
            throw new IllegalArgumentException(ERR_INVALID_DGM);
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
